"""A number of fallback implementations for backward compatibility"""

class dummy_set(object):
    """a minimal sufficient set implementation. Dumb and suboptimal"""
    def __init__(self, it = []):
        """constructs a new set object. optional it argument is an
        iterator which elements will be used as initial set content"""
        self._items = {}
        # since dict.fromkeys isn't available for py2.2
        # perform the task in a dumb way
        for el in it:
            self._items[el] = 1

    def __len__(self):
        return len(self._items)

    def __iter__(self):
        return iter(self._items)

    def __contains__(self, el):
        return self._items.__contains__(el)

    def add(self, el):
        if not self._items.has_key(el):
            self._items[el] = 1

    def clear(self):
        self._items = {}

    def copy(self):
        return self.__class__(self._items.keys())

    def difference(self, other):
        r = self.__class__()
        for el in self:
            if not el in other:
                r.add(el)
        return r

    def difference_update(self, other):
        for el in other:
            self.discard(el)
        return None

    def discard(self, el):
        try:
            self.remove(el)
        except KeyError:
            pass

    def intersection(self, other):
        r = self.__class__()
        for el in self:
            if el in other:
                r.add(el)
        return r

    def intersection_update(self, other):
        for el in self:
            if not el in other:
                self.discard(el)

    def issubset(self, other):
        for el in self:
            if not el in other:
                return False
        return True

    def issuperset(self, other):
        return other.issubset(self)

    def pop(self):
        return self._items.popitem()[0]

    def remove(self, el):
        del self._items[el]

    def symmetric_difference(self, other):
        r = self.__class__()
        for el in self:
            if not el in other:
                r.add(el)
        for el in other:
            if not el in self:
                r.add(el)
        return r

    def symmetric_difference_update(self, other):
        for el in self:
            if el in other:
                self.discard(el)
        for el in other:
            if not el in self:
                self.add(el)

    def union(self, other):
        r = self.copy()
        r.update(other)
        return r

    def update(self, other):
        for el in other:
            self.add(el)

    def __eq__(self, other):
        return self.issubset(other) and other.issubset(self)

    def __ne__(self, other):
        return not self.issubset(other) or not other.issubset(self)

    def __cmp__(self, other):
        raise TypeError('cannot compare sets using cmp()')

    def __str__(self):
        return str(repr(self))

    def __repr__(self):
        return "%s(%s)" % (self.__class__.__name__, repr(self._items.keys()))
# endclass dummy_set
