# -*- coding: utf-8; mode: python -*-
"""This module provides a fallback services for logging facilities"""

import sys
from traceback import format_exception

CRITICAL = 50
ERROR    = 40
WARNING  = 30
INFO     = 20
DEBUG    = 10
NOTSET   = 0

def __logmsg(prefix, msg, args, kwargs):
    args = ''.join(map(str, args))
    if kwargs.has_key('exc_info'):
        args += '\n' + ''.join(format_exception(*sys.exc_info()))
    if args:
        args = " " + args
    print >>sys.stderr, "%s:root:%s%s" % (prefix, msg, args)

def debug(msg, *args, **kwargs):
    return __logmsg("DEBUG", msg, args, kwargs)

def info(msg, *args, **kwargs):
    return __logmsg("INFO", msg, args, kwargs)

def warning(msg, *args, **kwargs):
    return __logmsg("WARNING", msg, args, kwargs)

def error(msg, *args, **kwargs):
    return __logmsg("ERROR", msg, args, kwargs)

def critical(msg, *args, **kwargs):
    return __logmsg("CRITICAL", msg, args, kwargs)

def exception(msg, *args):
    return error(msg, exc_info = 1, *args)

class _DummyLogger(object):
    def setLevel(self, _): pass

    debug     = staticmethod(debug)
    info      = staticmethod(info)
    warning   = staticmethod(warning)
    error     = staticmethod(error)
    critical  = staticmethod(critical)
    exception = staticmethod(exception)
# endclass _DummyLogger
    
    

_theDummyLogger = _DummyLogger()

def getLogger(_ = None):
    return _theDummyLogger
        
##    def log(level, msg, *args, **kwargs):
##        pass

